class Messages(object):
    """ Transactional emails are sometimes called triggered emails. Unlike bulk emails,
    they are sent one at the time on per need basis and contain highly personalized content.
    Examples of triggered emails can be one-off messages, such as password reset,
    statement generated, etc.

    Sending Transactional emails requires that newsletter templates for these emails are created
    prior to sending. Such template can have merge fields, in a format ${field_name}.
    This feature allows a high degree of flexibility for message customization.

    The newsletter to be sent can have a number of merge fields, with data for merging dynamically
    provided during a call.
    """

    endpoint = "messages"

    def __init__(self, ep):
        self.ep = ep

    def send_message(self, template_id, to, reply_to, from_name, subject, merge_fields=None, view_online=False,
                     click_tracking=True):
        """ Send s single transactional message.
        It is possible to inject HTML chunks into specific placeholders inside your email template.
        NOTE: It is important to use only single quotes in injected HTML

        :param to: Email address to send message to.
        :type to: str

        :param reply_to: This parameter specifies the email address which will be getting replies from your recipients
        should they choose to reply. The reply_to should be a valid email address.
        :type reply_to: str

        :param from_name: This parameter is displayed as "From" field in the email program when your recipients view
        your message. Use this value to clearly state your name or name of your organization.
        :type from_name: str

        :param subject: The subject of a transactional message
        :type subject: str

        :param merge_fields: Values for merge fields.
        :type merge_fields: dict

        :param view_online: Generates online version of sent message. We will host this generated message on our servers
        :type view_online: bool

        :param click_tracking: Overwrites all URLs in email to point to http://clicks.expresspigeon.com for
        click tracking. Setting it to false will preserve all URLs intact, but click tracking will not be available,
        default is true
        :type click_tracking: bool

        :returns: EpResponse with the id represents an ID of a message that was sent.
        You can use this value in order to get a report on status of this message.
        :rtype: EpResponse
        """

        return self.ep.post(self.endpoint, params={'template_id': template_id, 'to': to,
                                                   'reply_to': reply_to, 'from': from_name,
                                                   'subject': subject,
                                                   'merge_fields': merge_fields,
                                                   'view_online': view_online,
                                                   'click_tracking': click_tracking})

    def report(self, message_id):
        """ Returns a report with properties of a sent message, such as 'delivered' or 'bounced', 'opened', 'clicked'
        only if these events occurred.

        :param message_id: An ID of a message that was sent.
        :type message_id: str

        :returns: EpResponse with a report, e.g.
        {
           "id": 1,
           "email": "john@example.net",
           "in_transit": false,
           "delivered": true,
           "bounced": false,
           "opened": true,
           "clicked": true,
           "urls": [
             "http://example.net/buy_a_burger"
           ],
           "spam": false,
           "created_at": "2013-03-15T11:20:21.770+0000",
           "updated_at": "2013-03-16T11:22:23.210+0000"
        }
        :rtype: EpResponse
        """

        return self.ep.get("{0}/{1}".format(self.endpoint, message_id))

    def reports(self, start_date=None, end_date=None, from_id=None):
        """ Returns a report for at most 1000 of transactional emails sent with this API.
        The start_date and end_date parameters should be provided together.

        :param from_id: Id from where to get the next batch, e.g. the last id from the report.
        :type from_id: str

        :param start_date: Start of the reporting period (UTC, example 2013-03-16T11:22:23.210+0000)
        :type start_date: str

        :param end_date: End of the reporting period (UTC, example 2013-03-16T11:22:23.210+0000)
        :type end_date: str

        :returns: EpResponse with array of reports, e.g.
        [{
           "id": 1,
           "email": "john@example.net",
           "in_transit": false,
           "delivered": true,
           "bounced": false,
           "opened": true,
           "clicked": true,
           "urls": [
             "http://example.net/buy_a_burger"
           ],
           "spam": false,
           "created_at": "2013-03-15T11:20:21.770+0000",
           "updated_at": "2013-03-16T11:22:23.210+0000"
          },{
           "id": 2,
           "email": "bob@example.net",
           "in_transit": false,
           "delivered": true,
           "bounced": false,
           "opened": false,
           "clicked": false,
           "urls": [],
           "spam": false,
           "created_at": "2013-04-15T11:20:21.770+0000",
           "updated_at": "2013-04-16T11:22:23.210+0000"
        }]
        :rtype: EpResponse
        """

        params = []
        if from_id is not None:
            params.append("from_id=" + from_id)
        if start_date is not None and end_date is not None:
            params.append("start_date=" + start_date)
            params.append("end_date=" + end_date)
        query = self.endpoint
        if params:
            query += "?" + "&".join(params)
        return self.ep.get(query)


import os
import json
from collections import namedtuple
from expresspigeon.autoresponders import AutoResponders
from expresspigeon.campaigns import Campaigns
from expresspigeon.contacts import Contacts
from expresspigeon.lists import Lists
from expresspigeon.messages import Messages
from expresspigeon.templates import Templates

try:
    from urllib import request as url_lib
except ImportError:
    import urllib2 as url_lib


class InvalidAuthKey(Exception):
    pass


class ExpressPigeonException(Exception):
    pass


class ExpressPigeon(object):
    ROOT = "https://api.expresspigeon.com/"

    class Request(url_lib.Request):
        METHODS = ["get", "post", "put", "delete"]

        def __init__(self, url, method=None, headers=None, data=None, origin_req_host=None, unverifiable=False):
            if not headers:
                headers = {}
            url_lib.Request.__init__(self, url, data, headers, origin_req_host, unverifiable)
            self.method = method

        def get_method(self):
            if self.method:
                return self.method

            return url_lib.Request.get_method(self)

    def __init__(self, auth_key=None):
        """ Initialize the ExpressPigeon API client.

        :param auth_key: ExpressPigeon API key.  If not provided, the API key will be acquired from
        the EXPRESSPIGEON_AUTH_KEY environment variable.
        :type auth_key: string

        :returns: ExpressPigeon object for query API
        :rtype: ExpressPigeon

        :raises: :py:class:`InvalidAuthKey`: if the ExpressPigeon API key not found
        """
        if auth_key is None:
            if 'EXPRESSPIGEON_AUTH_KEY' in os.environ:
                auth_key = os.environ['EXPRESSPIGEON_AUTH_KEY']
            else:
                raise InvalidAuthKey('You must provide a ExpressPigeon API key')

        self.auth_key = auth_key
        self.lists = Lists(self)
        self.contacts = Contacts(self)
        self.campaigns = Campaigns(self)
        self.messages = Messages(self)
        self.templates = Templates(self)
        self.auto_responders = AutoResponders(self)

    def __getattr__(self, name):
        """
        This is metaprogramming trick to send get, post, put, delete requests implicitly
        """
        return (
            lambda endpoint, **kwargs: self.__send_request__(endpoint, name, **kwargs)
            if name in self.Request.METHODS
            else super(ExpressPigeon, self).__getattribute__(name)
        )

    def __send_request__(self, endpoint, method, **kwargs):
        content_type = kwargs["content_type"] if "content_type" in kwargs else "application/json"
        body = kwargs["body"] if "body" in kwargs else json.dumps(kwargs["params"] if "params" in kwargs else {})

        opener = url_lib.build_opener(url_lib.HTTPSHandler)

        req = self.Request(url=(self.ROOT if self.ROOT.endswith("/") else self.ROOT + "/") + endpoint,
                           method=method.upper(),
                           headers={"X-auth-key": self.auth_key, "Content-type": content_type},
                           data=body.encode("utf-8"))

        self.request_hook(req)

        try:
            return json.loads(opener.open(req).read().decode("utf-8"), "UTF-8",
                              object_hook=lambda d: namedtuple('EpResponse', d.keys())(*d.values()))
        except url_lib.HTTPError as e:
            return json.loads(e.fp.read().decode("utf-8"), "UTF-8",
                              object_hook=lambda d: namedtuple('EpResponse', d.keys())(*d.values()))

    def read_stream(self, endpoint, **kwargs):
        opener = url_lib.build_opener(url_lib.HTTPSHandler)

        req = self.Request(url=(self.ROOT if self.ROOT.endswith("/") else self.ROOT + "/") + endpoint,
                           method="GET", headers={"X-auth-key": self.auth_key})

        self.request_hook(req)

        try:
            return opener.open(req).read().decode("utf-8")
        except url_lib.HTTPError as e:
            return json.loads(e.fp.read().decode("utf-8"), "UTF-8",
                              object_hook=lambda d: namedtuple('EpResponse', d.keys())(*d.values()))

    def request_hook(self, request):
        pass


class Campaigns(object):
    """Campaigns endpoint
    """

    endpoint = "campaigns"

    def __init__(self, ep):
        self.ep = ep

    def get_all(self, start_date=None, end_date=None, from_id=None):
        """ Returns up to 1000 campaigns starting with smallest campaign id in date range.

        :param from_id: smallest campaign id
        :type from_id: str

        :param start_date: Start of the reporting period (UTC, for example 2013-03-16T11:22:23.210+0000)
        :type start_date: str

        :param end_date: End of the reporting period (UTC, for example 2013-03-16T11:22:23.210+0000)
        :type end_date: str

        :returns: list of campaigns EPResponse objects, e.g.
        [{
            "total":1,
            "id":1,
            "send_time":"2014-01-10T10:24:22.000+0000",
            "template_name":"1",
            "reply_to":"john@example.net",
            "from_name":"John",
            "subject":"Hi",
            "name":"My Campaign",
            "list_id":1
        }]
        :rtype: list of EpResponse objects
        """

        params = []
        if from_id is not None:
            params.append("from_id=" + str(from_id))
        if start_date is not None and end_date is not None:
            params.append("start_date=" + start_date)
            params.append("end_date=" + end_date)
        query = self.endpoint
        if params:
            query += "?" + "&".join(params)
        return self.ep.get(query)

    def send(self, list_id, template_id, name, from_name, reply_to, subject, google_analytics):
        """ Creates a campaign. Invocation of this API will trigger sending a new campaign.
        The content type of a request must be application/json.

        :param list_id: The id of a list the campaign is sent to. The list must be enabled.
        :type list_id: long

        :param template_id: The id of a newsletter template used for the campaign.
        :type template_id: long

        :param name: The name of a campaign. This name is for your reference only and will not be exposed
        to your audience. If you have Google Analytics turned on, this value will also be used for
        Google Analytics campaign.
        :type name: str

        :param from_name: This parameter is displayed as "From" field in the email program when your recipients view
        your message. Use this value to clearly state your name or name of your organization.
        :type from_name: str

        :param reply_to: This parameter specifies the email address which will be getting replies from your recipients
        should they choose to reply. The reply_to should be a valid email address.
        :type reply_to: str

        :param subject: The subject of a newsletter
        :type subject: str

        :param google_analytics: Indicates whether Google Analytics should be enabled for a campaign.
        Should be true or false.
        :type google_analytics: bool

        :returns: EpResponse with campaign_id field
        :rtype: EpResponse
        """
        return self.ep.post(self.endpoint, params={'list_id': list_id, 'template_id': template_id, 'name': name,
                                                   'from_name': from_name, 'reply_to': reply_to, 'subject': subject,
                                                   'google_analytics': google_analytics})

    def schedule(self, list_id, template_id, name, from_name, reply_to, subject, google_analytics, schedule_for):
        """ Schedules a campaign. The content type of a request must be application/json.

        :param list_id: The id of a list the campaign is sent to. The list must be enabled.
        :type list_id: long

        :param template_id: The id of a newsletter template used for the campaign.
        :type template_id: long

        :param name: The name of a campaign. This name is for your reference only and will not be exposed
        to your audience. If you have Google Analytics turned on, this value will also be used
        for Google Analytics campaign.
        :type name: str

        :param from_name: This parameter is displayed as "From" field in the email program when your recipients view
        your message. Use this value to clearly state your name or name of your organization.
        :type from_name: str

        :param reply_to: This parameter specifies the email address which will be getting replies from your recipients
        should they choose to reply. The reply_to should be a valid email address.
        :type reply_to: str

        :param subject: The subject of a newsletter
        :type subject: str

        :param google_analytics: Indicates whether Google Analytics should be enabled for a campaign.
        Should be true or false.
        :type google_analytics: bool

        :param schedule_for: Specifies what time a campaign should be sent. The schedule_for must be in ISO date format
        and should be in the future.
        :type schedule_for: str

        :returns: EpResponse with campaign_id field
        :rtype: EpResponse
        """

        if schedule_for is None or schedule_for == "":
            raise Exception("schedule_for cannot be None or empty")

        return self.ep.post(self.endpoint, params={'list_id': list_id, 'template_id': template_id, 'name': name,
                                                   'from_name': from_name, 'reply_to': reply_to, 'subject': subject,
                                                   'google_analytics': google_analytics, 'schedule_for': schedule_for})

    def report(self, campaign_id):
        """ Returns report for specific campaign

        :param campaign_id: Campaign ID the report is generated for
        :type campaign_id: long

        :returns: report dict, e.g. {delivered=0, clicked=0, opened=0, spam=0, in_transit=1, unsubscribed=0, bounced=0}
        :rtype: EpResponse
        """

        return self.ep.get("{0}/{1}".format(self.endpoint, campaign_id))

    def bounced(self, campaign_id):
        """ Returns bounced contacts

        :param campaign_id: Campaign ID bounced contacts is found for
        :type campaign_id: long

        :returns: list of dicts, e.g. [{'id': 1, 'email': 'a@a.a'}]
        :rtype: EpResponse
        """

        return self.ep.get("{0}/{1}/bounced".format(self.endpoint, campaign_id))

    def unsubscribed(self, campaign_id):
        """ Returns unsubscribed contacts

        :param campaign_id: Campaign ID unsubscribed contacts is found for
        :type campaign_id: long

        :returns: list of dicts, e.g. [{'id': 1, 'email': 'a@a.a'}]
        :rtype: EpResponse
        """

        return self.ep.get("{0}/{1}/unsubscribed".format(self.endpoint, campaign_id))

    def spam(self, campaign_id):
        """ Returns contacts that mark campaign as spam

        :param campaign_id: Campaign ID spam contacts is found for
        :type campaign_id: long

        :returns: list of dicts, e.g. [{'id': 1, 'email': 'a@a.a'}]
        :rtype: EpResponse
        """

        return self.ep.get("{0}/{1}/spam".format(self.endpoint, campaign_id))



class Contacts(object):
    """Contacts endpoint
    """
    endpoint = "contacts"

    def __init__(self, ep):
        self.ep = ep

    def upsert(self, list_id, contacts):
        """ JSON document represents a list of contacts to be created or updated.
        The email field is required.
        When updating a contact, list_id is optional,
        since the contact is uniquely identified by email across all lists.

        :param list_id: Contact list ID the contact will be added to
        :type list_id: long

        :param contacts: List of dictionaries describes new contacts. The "email" field is required.
        :type contacts: list

        :returns: EpResponse with JSON representation of a contact
        :rtype: EpResponse
        """
        return self.ep.post(self.endpoint, params={"list_id": list_id, "contacts": contacts})

    def find_by_email(self, email):
        """ Returns a single contact by email address.

        :param email: Email of contact to be selected.
        :type email: str

        :returns: EpResponse with all contact fields, e.g.
        {
             "custom_fields": {
                 "my custom field": "custom value"
             },
            "first_name": "Bob",
            "email": "bob@example.net",
           "email_format": "html",
           "created_at": "2012-10-29T14:17:58.000+0000",
           "updated_at": "2013-01-24T08:20:52.000+0000",
           "status": "ENGAGED"
           "lists": [
             {
               "id": 1
             },
             {
               "id": 2
             }]
        }
        :rtype: EpResponse
        """

        return self.ep.get("{0}?email={1}".format(self.endpoint, email))

    def delete(self, email, list_id=None):
        """ Delete single contact. If list_id is not provided, contact will be deleted from system.

        :param email: contact email to be deleted.
        :type email: str

        :param list_id: list id to remove contact from, if not provided, contact will be deleted from system.
        :type list_id: long

        """

        query = "{0}?email={1}".format(self.endpoint, email) if list_id is None \
            else "{0}?email={1}&list_id={2}".format(self.endpoint, email, list_id)
        return self.ep.delete(query)



class AutoResponders(object):
    """AutoResponders endpoint
    """

    endpoint = "auto_responders"

    def __init__(self, ep):
        self.ep = ep

    def start(self, autoresponder_id, email):
        """ This feature allow developers to start auto responder for a contact.

        :param autoresponder_id: auto responder id to be started for a contact
        :type autoresponder_id: int

        :param email: contact email
        :type email: str

        :returns: EpResponse with message about autoresponder start
        :rtype: EpResponse
        """
        return self.ep.post("{0}/{1}/start".format(self.endpoint, autoresponder_id), params={"email": email})

    def stop(self, autoresponder_id, email):
        """ This feature allow developers to stop auto responder for a contact.

        :param autoresponder_id: auto responder id to be stopped for a contact
        :type autoresponder_id: int

        :param email: contact email
        :type email: str

        :returns: EpResponse with message about autoresponder stop
        :rtype: EpResponse
        """
        return self.ep.post("{0}/{1}/stop".format(self.endpoint, autoresponder_id), params={"email": email})

    def find_all(self):
        """ Returns all auto responders.

        :returns: list of auto responders, e.g.
        [
            {
                "auto_responder_parts": [
                    {
                        "auto_responder_part_id": 9,
                        "subject": "Test autoresponder",
                        "template_id": 347
                    }
                ],
                "name": "Test",
                "auto_responder_id": 9
            }
        ]
        :rtype: list
        """

        return self.ep.get(self.endpoint)

    def report(self, autoresponder_id):
        """ Returns report for auto responder

        :param autoresponder_id: auto responder id the report is generated for
        :type autoresponder_id: long

        :returns: list of EpResponse with report dict, e.g.
        [{delivered=4, clicked=0, opened=0, spam=0, in_transit=0, unsubscribed=0, bounced=0, auto_responder_part_id=9}]
        :rtype: list of EpResponse
        """

        return self.ep.get("{0}/{1}".format(self.endpoint, autoresponder_id))

    def bounced(self, autoresponder_id, auto_responder_part_id):
        """ Returns a list of object(s) with email and id of bounced contacts
        associated with given autoresponder part.

        :param autoresponder_id: auto responder id the bounced contacts are found for
        :type autoresponder_id: long

        :param auto_responder_part_id: autoresponder part id the bounced contacts are found for
        :type auto_responder_part_id: long

        :returns: list of of object(s) with email and id
        :rtype: list
        """
        return self.ep.get("{0}/{1}/{2}/bounced".format(self.endpoint, autoresponder_id, auto_responder_part_id))

    def unsubscribed(self, autoresponder_id, auto_responder_part_id):
        """ Returns an array of object(s) with email and id of unsubscribed contacts
        associate with this autoresponder part.

        :param autoresponder_id: auto responder id the unsubscribed contacts are found for
        :type autoresponder_id: long

        :param auto_responder_part_id: autoresponder part id the bounced contacts are found for
        :type auto_responder_part_id: long

        :returns: list of contacts ids
        :rtype: list
        """

        return self.ep.get("{0}/{1}/{2}/unsubscribed".format(self.endpoint, autoresponder_id, auto_responder_part_id))

    def spam(self, autoresponder_id, auto_responder_part_id):
        """ Returns an array of object(s) with email and id of spam contacts associated with this autoresponder part.

        :param autoresponder_id: auto responder id the spam contacts are found for
        :type autoresponder_id: long

        :param auto_responder_part_id: autoresponder part id the bounced contacts are found for
        :type auto_responder_part_id: long

        :returns: list of contacts ids
        :rtype: list
        """

        return self.ep.get("{0}/{1}/{2}/spam".format(self.endpoint, autoresponder_id, auto_responder_part_id))


class Templates(object):
    """
    Templates endpoint
    """

    endpoint = "templates"

    def __init__(self, ep):
        self.ep = ep

    def copy(self, template_id, name, merge_fields=None):
        """ This feature allow developers to create a copy of an email template and at the same time merge data into
         a new version. It makes it possible to have the following workflow:

        1. Create a blank newsletter and add merge fields to it (using email editor)
        2. Make a new copy of this newsletter, and merge specific data into it using this API
        3. Send or schedule a new campaign with the API
        Steps 2 nd 3 can be done remotely with the API, without having to log into the website.
        Combined with ability to create new lists on the fly, and upload contacts,
        it provides an opportunity to build powerful marketing solutions.

        :param template_id: template id to be used as a source
        :type template_id: int

        :param name: name for a new template
        :type name: str

        :param merge_fields: values of merge fields
        :type merge_fields: dict

        :returns: EpResponse with 'template_id' field
        :rtype: EpResponse
        """
        return self.ep.post("{0}/{1}/copy".format(self.endpoint, template_id),
                            params={"name": name, "merge_fields": merge_fields})


import os
import random
import string


class Lists(object):
    """
    Lists endpoint
    """

    endpoint = "lists"

    def __init__(self, ep):
        self.ep = ep

    def find_all(self):
        """ Query all lists

        :returns: List of JSON of all lists for user
        :rtype: EpResponse
        """

        return self.ep.get(self.endpoint)

    def create(self, name, from_name, reply_to):
        """ Create a list

        :param name: Name of a newly created list
        :type name: str

        :param from_name: Default "from" name used when sending campaigns to this list
        :type from_name: str

        :param reply_to: Default Reply To email address used when sending campaigns to this list
        :type reply_to: str

        :returns: ContactList object
        :rtype: EpResponse
        """

        params = {"name": name, "from_name": from_name, "reply_to": reply_to}
        return self.ep.post(self.endpoint, params=params)

    def delete(self, list_id):
        """ Removes a list with a given id.
            A list must be enabled and has no dependent subscriptions and/or scheduled campaigns.

        :param list_id: Id of list to be removed
        :type list_id: int

        :returns: EpResponse with status, code, and message
        :rtype: EpResponse

        """
        return self.ep.delete('{0}/{1}'.format(self.endpoint, list_id))

    def update(self, list_id, params):
        """ Updates existing list

            :param list_id: Id of list to be updated
            :type list_id: int

            :param params: JSON object represents a list to be updated
            :type params: dict

            :returns: EpResponse with status, code, message, and updated list
            :rtype: EpResponse
        """
        params['id'] = list_id
        return self.ep.put(self.endpoint, params=params)

    def upload(self, list_id, contacts_file):
        """ Creates or merges contacts from uploaded CSV file.

        :param list_id: Id of list to be updated with contacts from CSV
        :type list_id: int

        :param contacts_file: Absolute path to the CSV file with contacts
        :type contacts_file: str

        :returns: EpResponse with status, code, and upload_id field
        :rtype: EpResponse
        """

        with (open(contacts_file)) as f:
            boundary = ''.join(random.choice(string.digits + string.ascii_letters) for i in range(30))
            lines = []
            lines.extend((
                '--{0}'.format(boundary),
                'Content-Disposition: form-data; name="contacts_file"; contacts_file="{0}"'
                .format(os.path.basename(contacts_file)),
                '',
                str(f.read())
            ))

            lines.extend((
                '--{0}--'.format(boundary),
                '',
            ))
            body = '\r\n'.join(lines)

            return self.ep.post('{0}/{1}/upload'.format(self.endpoint, list_id),
                                content_type="multipart/form-data; boundary={0}".format(boundary), body=body)

    def upload_status(self, upload_id):
        """ Checks status of upload. If the upload was finished a detailed report is returned.

        :param upload_id: The id of a list uploaded(this id was returned from :func:`upload`).

        :returns: EpResponse with status, code, and "report" field, e.g:
            "report": {
                        "suppressed": 0,
                        "skipped": 0,
                        "list_name": "Active customers",
                        "merged": 1,
                        "imported": 1
                    }
        :rtype: EpResponse
        """
        return self.ep.get('{0}/{1}/{2}'.format(self.endpoint, 'upload_status', upload_id))

    def csv(self, list_id):
        """ Returns contacts as CSV

        :param list_id: list id to export as csv
        :type list_id: str

        :returns: response as is
        :rtype: str or EpResponse
        """

        return self.ep.read_stream("{0}/{1}/csv".format(self.endpoint, list_id))


import os
from setuptools import setup, find_packages

long_description = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

setup(name='ExpressPigeon',
      version='0.0.4',
      description='ExpressPigeon API',
      long_description=long_description,
      author='Gleb Galkin',
      author_email='gleb@expresspigeon.com',
      license='BSD',
      packages=find_packages(exclude=['*.tests', '*.tests.*', 'tests.*', 'tests']),

      keywords=' '.join(['expresspigeon',
                         'api',
                         'email',
                         'marketing',
                         'transactional'
      ]),
      url='https://github.com/expresspigeon/expresspigeon-python',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Programming Language :: Python',
          'Topic :: Communications :: Email',
          'Topic :: Software Development :: Libraries',
          'Topic :: Software Development :: Libraries :: Python Modules'
      ],
      tests_require=['pytz'])